/*!
  * Bootstrap button.js v1.0.6 (https://mykademy.com/)
  * Copyright 2011-2022 Sajan Christudas, Sajan Gurung, Bicky, Prabina
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
  */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('./util/index'), require('./dom/event-handler'), require('./base-component')) :
  typeof define === 'function' && define.amd ? define(['./util/index', './dom/event-handler', './base-component'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.Button = factory(global.Index, global.EventHandler, global.BaseComponent));
}(this, (function (index, EventHandler, BaseComponent) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var EventHandler__default = /*#__PURE__*/_interopDefaultLegacy(EventHandler);
  var BaseComponent__default = /*#__PURE__*/_interopDefaultLegacy(BaseComponent);

  /**
   * --------------------------------------------------------------------------
   * Bootstrap (v5.0.2): button.js
   * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
   * --------------------------------------------------------------------------
   */
  /**
   * ------------------------------------------------------------------------
   * Constants
   * ------------------------------------------------------------------------
   */

  const NAME = 'button';
  const DATA_KEY = 'bs.button';
  const EVENT_KEY = `.${DATA_KEY}`;
  const DATA_API_KEY = '.data-api';
  const CLASS_NAME_ACTIVE = 'active';
  const SELECTOR_DATA_TOGGLE = '[data-bs-toggle="button"]';
  const EVENT_CLICK_DATA_API = `click${EVENT_KEY}${DATA_API_KEY}`;
  /**
   * ------------------------------------------------------------------------
   * Class Definition
   * ------------------------------------------------------------------------
   */

  class Button extends BaseComponent__default['default'] {
    // Getters
    static get NAME() {
      return NAME;
    } // Public


    toggle() {
      // Toggle class and sync the `aria-pressed` attribute with the return value of the `.toggle()` method
      this._element.setAttribute('aria-pressed', this._element.classList.toggle(CLASS_NAME_ACTIVE));
    } // Static


    static jQueryInterface(config) {
      return this.each(function () {
        const data = Button.getOrCreateInstance(this);

        if (config === 'toggle') {
          data[config]();
        }
      });
    }

  }
  /**
   * ------------------------------------------------------------------------
   * Data Api implementation
   * ------------------------------------------------------------------------
   */


  EventHandler__default['default'].on(document, EVENT_CLICK_DATA_API, SELECTOR_DATA_TOGGLE, event => {
    event.preventDefault();
    const button = event.target.closest(SELECTOR_DATA_TOGGLE);
    const data = Button.getOrCreateInstance(button);
    data.toggle();
  });
  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   * add .Button to jQuery only if jQuery is present
   */

  index.defineJQueryPlugin(Button);
  /**
   * ------------------------------------------------------------------------
   * Ripple effect
   * ------------------------------------------------------------------------
   */

  document.addEventListener('mousedown', e => {

    if (e.target.classList.contains('btn')) {
      const button = e.target;
      const rippleEffect = button.querySelector('.ripple-effect');

      if (rippleEffect) {
        rippleEffect.remove();
      } // var posX = button.offsetLeft
      // var posY = button.offsetTop


      let buttonWidth = button.offsetWidth;
      let buttonHeight = button.offsetHeight;
      const span = document.createElement('span');

      if (button.classList.contains('dark')) {
        span.classList.add('ripple-effect-dark');
      } else {
        span.classList.add('ripple-effect');
      }

      button.appendChild(span);

      if (buttonWidth >= buttonHeight) {
        buttonHeight = buttonWidth;
      } else {
        buttonWidth = buttonHeight;
      }

      const rect = e.target.getBoundingClientRect();
      const x = e.clientX - rect.left - buttonWidth / 2; // x position within the element.

      const y = e.clientY - rect.top - buttonHeight / 2; // var x = e.pageX - posX - (buttonWidth / 2)
      // var y = e.pageY - posY - (buttonHeight / 2)

      span.style.width = buttonWidth + 'px';
      span.style.height = buttonHeight + 'px';
      span.style.left = x + 'px';
      span.style.top = y + 'px';
      span.classList.add('ripple-animation');
      span.addEventListener('webkitAnimationEnd', e => {
        e.target.remove();
      }); // setTimeout(span.remove(), 4000)
    } // Clear Search text


    if (e.target.classList.contains('btn-clear')) {
      const search = e.target.parentNode;

      if (e.target.parentNode.classList.contains('search-wrapper')) {
        search.querySelector('input').value = '';
      }
    }
  });

  return Button;

})));
//# sourceMappingURL=button.js.map
